/*
;--- TWI settings --------------------------------------------------
.equ	TWI_PORT			= PORTC
.equ	TWI_SCL				= PC5
.equ	TWI_SDA				= PC4
.equ	F_TWI				= 400000		; Hz
*/



;--- TWI-Berechnungen ---------------------------------------------
.equ		c1BAUD_TWI	= ((F_CPU/F_TWI)-16)/2
.equ		c2BAUD_TWI	= c1BAUD_TWI/4
.equ		c3BAUD_TWI	= c2BAUD_TWI/4
.equ		c4BAUD_TWI	= c3BAUD_TWI/4
.if		c1BAUD_TWI <= 255
.equ	BAUD_TWI = c1BAUD_TWI
.equ	TWPS_bm = 0
.elif	c2BAUD_TWI <= 255
.set	BAUD_TWI = c2BAUD_TWI
.equ	TWPS_bm = 1
.elif	c3BAUD_TWI <= 255
.equ	BAUD_TWI = c3BAUD_TWI
.equ	TWPS_bm = 2
.else
.equ	BAUD_TWI = c4BAUD_TWI
.equ	TWPS_bm = 3
.endif


.equ	twiSTART		= 0x08		; Start
.equ	twiRESTART		= 0x10		; Repeated Start 
.equ	twiSwACK		= 0x18		; Start WR + ACK
.equ	twiSrACK		= 0x40		; Start RD + ACK
.equ	twiSwNACK		= 0x20		; Start WR + NACK		--> Error
.equ	twiSrNACK		= 0x48		; Start RD + NACK		--> Error
.equ	twiDwACK		= 0x28		; Data WR + ACK
.equ	twiDrACK		= 0x50		; Data RD + ACK
.equ	twiDwNACK		= 0x30		; Data WR + NACK		--> Error
.equ	twiDrNACK		= 0x58		; Data RD + NACK		--> Last Byte Read; Stop follow

;.def		TWI_STATE	= GPIOR2
.equ		_TWI_ERR	= 0


;------------------------------------------------------------------------------------------------------------------------------------------------------
init_twi:
		clr		r16
;		out		GPIOR2,r16
;---- Set TWI Baudrate ---------------------------------------------------------------
		ldi		r16,(BAUD_TWI)
		out		(TWBR),r16
;---- Set TWI Prescaller ---------------------------------------------------------------
		ldi 	r16, (TWPS_bm <<0)
		out 	TWSR, r16		
;---- Enable TWI ---------------------------------------------------------------------
		ldi 	r16, 0<<TWINT|1<<TWEN
		out 	TWCR,r16
		ret				
;------------------------------------------------------------------------------------------------------------------------------------------------------		
twi_send_rd_start:
		set
		rjmp	PC+2
twi_send_wr_start:
		clt
		ldi 	r16, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi 	r16,twiSTART
		breq	PC+3
		cpi		r16,twiRESTART
		brne	twi_error_handler
		mov		r16,i2c_addr
		bld		r16,0					
		out 	TWDR, r16
		ldi 	r16, (1<<TWINT) | (1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi 	r16,twiSwACK
		breq	PC+3
		cpi		r16,twiSrACK
		brne	twi_error_handler		
		ret
;------------------------------------------------------------------------------------------------------------------------------------------------------
twi_write:
		out 	TWDR, r16
twi_write_wait:
		ldi 	r16, (1<<TWINT) | (1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi 	r16,twiDwACK
		brne	twi_error_handler
		ret
;------------------------------------------------------------------------------------------------------------------------------------------------------		
twi_read_nack:
		ldi 	r16, (1<<TWINT)|(0<<TWEA)|(1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2	
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi 	r16,twiDrNACK
		brne	twi_error_handler
		in		r17,(TWDR)	
; stop
		ldi 	r16, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
		out 	TWCR, r16
		ret
;------------------------------------------------------------------------------------------------------------------------------------------------------
twi_read_ack:
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		ldi 	r16, (1<<TWINT)|(1<<TWEA)|(1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi 	r16,twiDrACK
		brne	twi_error_handler
		in		r17,(TWDR)
		ret	
;------------------------------------------------------------------------------------------------------------------------------------------------------
twi_error_handler:
;		sbi		GPIOR2,_TWI_ERR
		rcall	twi_send_stop
; TWI aus				
		ldi 	r16, (1<<TWINT) | (0<<TWEN)
		out 	TWCR, r16
		ret	
;------------------------------------------------------------------------------------------------------------------------------------------------------
twi_read_bytes:		; scrached:	r16, r0
		out 	TWDR, r16
		ldi 	r16, (1<<TWINT) | (1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi 	r16,twiDwACK
		brne	twi_error_handler		
;--- restart v ------------- ^ write---------------
		ldi 	r16, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi		r16,twiRESTART
		brne	twi_error_handler
		mov		r16,i2c_addr
		inc		r16					
		out 	TWDR, r16
		ldi 	r16, (1<<TWINT) | (1<<TWEN)
		out 	TWCR, r16
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi		r16,twiSrACK
		brne	twi_error_handler
;--- read v ------------------ ^ restart -----------
twi_read_bytes_loop:
		cpi		r17,1
		brne	PC+3
		ldi 	r16, (1<<TWINT)|(0<<TWEA)|(1<<TWEN)		; last Byte was read
		rjmp	PC+2
		ldi 	r16, (1<<TWINT)|(1<<TWEA)|(1<<TWEN)
		out 	TWCR, r16
;-------				
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16, TWCR
		sbrs 	r16, TWINT
		rjmp	PC-2
		in 		r16,TWSR		
		andi 	r16, 0xF8
		cpi	 	r16,twiDrNACK
		breq	PC+3
		cpi	 	r16,twiDrACK
		brne	twi_error_handler
		in		r16,(TWDR)
		st		Y+,r16		
		dec		r17
		brne	twi_read_bytes_loop
; stop
		ldi 	r16, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
		out 	TWCR, r16
		ret
;------------------------------------------------------------------------------------------------------------------------------------------------------
twi_send_stop:
		ldi 	r16, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
		out 	TWCR, r16
		ret
;------------------------------------------------------------------------------------------------------------------------------------------------------