/*
 * mSSD1306_driver.inc
 *
 *  Created: 22.10.2020 15:39:45
 *   Author: Steffen Hörhold
 */ 

// SLA (0x3C) + WRITE_MODE (0x00) =  0x78 (0b 0111 1000)->(0011 1100) 0x3C
.equ	OLED_ADDR						= 0x3C
.equ	contrast						= 0xFF	; 0x7F
.equ	display_clk						= 0xF0	; 0x80
.equ	charge_pump						= 0x14


// Control byte
.equ	SSD1306_CONTROL_BYTE_CMD_SINGLE		= 0x80
.equ	SSD1306_CONTROL_BYTE_CMD_STREAM		= 0x00
.equ	SSD1306_CONTROL_BYTE_DATA_SINGLE	= 0xC0
.equ	SSD1306_CONTROL_BYTE_DATA_STREAM	= 0x40

// Fundamental commands (pg.28)
.equ	SSD1306_CMD_SET_CONTRAST			= 0x81    // follow with 0x7F
.equ	SSD1306_CMD_DISPLAY_RAM				= 0xA4
.equ	SSD1306_CMD_DISPLAY_ALLON			= 0xA5
.equ	SSD1306_CMD_DISPLAY_NORMAL			= 0xA6
.equ	SSD1306_CMD_DISPLAY_INVERTED		= 0xA7
.equ	SSD1306_CMD_DISPLAY_OFF				= 0xAE
.equ	SSD1306_CMD_DISPLAY_ON				= 0xAF

// Addressing Command Table (pg.30)
.equ	SSD1306_CMD_SET_LOWER_COLUMN		= 0x00
.equ	SSD1306_CMD_SET_HIGHER_COLUMN		= 0x10
.equ	SSD1306_CMD_SET_MEMORY_ADDR_MODE	= 0x20    // follow with 0x00 = HORZ mode = Behave like a KS108 graphic LCD
	.equ	HORIZONTAL_ADDRESSING_MODE		= 0
	.equ	VERTICAL_ADDRESSING_MODE		= 1
	.equ	PAGE_ADDRESSING_MODE			= 2

.equ	SSD1306_CMD_SET_PAGE_START_ADDRESS	= 0xB0    //~0xB7
.equ	SSD1306_CMD_SET_COLUMN_ADDRESS		= 0x21    // can be used only in HORZ/VERT mode - follow with 0x00 and 0x7F = COL127
.equ	SSD1306_CMD_SET_PAGE_ADDRESS		= 0x22    // can be used only in HORZ/VERT mode - follow with 0x00 and 0x07 = PAGE7

// Hardware Config (pg.31)
.equ	SSD1306_CMD_SET_DISPLAY_START_LINE	= 0x40
.equ	SSD1306_CMD_SET_SEGMENT_REMAP_01	= 0xA0    
.equ	SSD1306_CMD_SET_SEGMENT_REMAP_02	= 0xA1    
.equ	SSD1306_CMD_SET_MUX_RATIO			= 0xA8    // follow with 0x3F = 64 MUX
.equ	SSD1306_CMD_SET_COM_OUT_NORMAL		= 0xC0
.equ	SSD1306_CMD_SET_COM_OUT_REMAPPED	= 0xC8    
.equ	SSD1306_CMD_SET_DISPLAY_OFFSET		= 0xD3    // follow with 0x00
.equ	SSD1306_CMD_SET_COM_PIN_MAP			= 0xDA    // follow with 0x12
.equ	SSD1306_CMD_NOP						= 0xE3    // NOP

// Timing and Driving Scheme (pg.32)
.equ	SSD1306_CMD_SET_DISPLAY_CLK_DIV		= 0xD5    // follow with 0x80
.equ	SSD1306_CMD_SET_PRECHARGE			= 0xD9    // follow with 0xF1
.equ	SSD1306_CMD_SET_VCOMH_DESELCT		= 0xDB    // follow with 0x30

// Charge Pump (pg.62)
.equ	SSD1306_CMD_SET_CHARGE_PUMP			= 0x8D    // follow with 0x14


init_oled:
		delayms	20
		ldi		r16,OLED_ADDR
		lsl		r16
		mov		i2c_addr,r16
;i2c start
		rcall	twi_send_wr_start										; START + ADDR			
;i2c write Control Byte CMD STREAM
		ldi		r16,(SSD1306_CONTROL_BYTE_CMD_STREAM)					; CONTROL Byte: CMD STREAM
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_DISPLAY_OFF)							; DISPLAY_OFF
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_NOP)									; NOP
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_MEMORY_ADDR_MODE)					; SET_MEMORY_ADDR_MODE
		rcall	twi_write
;i2c write DATA
		ldi		r16,(HORIZONTAL_ADDRESSING_MODE)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_PAGE_START_ADDRESS)				; SET_PAGE_START_ADDRESS (0xB0)
		rcall	twi_write
;i2c write COMMAND
;		ldi		r16,(SSD1306_CMD_SET_COM_OUT_NORMAL)					; SET_COM_REMAPPED (0xC0)
		ldi		r16,(SSD1306_CMD_SET_COM_OUT_REMAPPED)					; SET_COM_REMAPPED (0xC8)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_LOWER_COLUMN)						; SET_LOWER_COLUMN (0x00) -> SEG0
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_HIGHER_COLUMN)						; SET_HIGHER_COLUMN (0x10) -> SEG0
		rcall	twi_write


;i2c write CMD
		ldi		r16,(SSD1306_CMD_SET_COLUMN_ADDRESS)				; SET_COLUMN_ADDRRSS
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0)												; Start Address
		rcall	twi_write
;i2c write DATA
		ldi		r16,(127)											; End Address
		rcall	twi_write

;i2c write CMD
		ldi		r16,(SSD1306_CMD_SET_PAGE_ADDRESS)					; SET_PAGE_ADDRRSS
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0)												; Start Address
		rcall	twi_write
;i2c write DATA
		ldi		r16,(7)												; End Address
		rcall	twi_write


;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_DISPLAY_START_LINE)				; SET_DISPLAY_START_LINE (0x40) -> 0
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_DISPLAY_RAM)							; DISPLAY_RAM
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_SEGMENT_REMAP_02)					; SET_SEGMENT_REMAP_02 (0xA1)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_DISPLAY_NORMAL)						; DISPLAY_NORMAL
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_CONTRAST)							; SET_CONTRAST
		rcall	twi_write
;i2c write DATA
		ldi		r16,(contrast)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_MUX_RATIO)							; SET_MUX_RATIO (0xA8)
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0x3F)												; 0x3F = 63
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_DISPLAY_OFFSET)					; SET_DISPLAY_OFFSET (0xD3)
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0x00)												; (0x00)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_DISPLAY_CLK_DIV)					; SET_DISPLAY_CLK_DIV
		rcall	twi_write
;i2c write DATA
		ldi		r16,(display_clk)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_PRECHARGE)							; SET_PRECHARGE
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0x22)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_COM_PIN_MAP)						; SET_COM_PIN_MAP (0xDA)
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0x12)												; (0x12)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_VCOMH_DESELCT)						; SET_VCOMH_DESELCT
		rcall	twi_write
;i2c write DATA
		ldi		r16,(0x20)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_SET_CHARGE_PUMP)						; SET_CHARGE_PUMP
		rcall	twi_write
;i2c write DATA
		ldi		r16,(charge_pump)
		rcall	twi_write
;i2c write COMMAND
		ldi		r16,(SSD1306_CMD_DISPLAY_ON)							; DISPLAY_ON
		rcall	twi_write
		rcall	twi_send_stop
		ret


oled_clr_screen:
		ldi		r16,OLED_ADDR
		lsl		r16
		mov		i2c_addr,r16
		clr		r17
oled_clr_loop_a:
		clr		r16
		rcall	oled_gotoxy
;i2c start
		rcall	twi_send_wr_start										; START + ADDR
;i2c write Control Byte -> DATA STREAM
		ldi		r16,(SSD1306_CONTROL_BYTE_DATA_STREAM)					; CONTROL Byte: DATA STREAM
		rcall	twi_write
		ldi		r18,(128)
oled_clr_loop_b:
;i2c write DATA
		clr		r16
		rcall	twi_write
		dec		r18
		brne	oled_clr_loop_b
		rcall	twi_send_stop
		inc		r17
		cpi		r17,8
		brne	oled_clr_loop_a
		rcall	twi_send_stop
		rcall	oled_home
		ret

oled_home:
		clr		r16
		clr		r17
		rcall	oled_gotoxy
		ret


oled_gotoxy:	; x = r16 [0..15]   |    y = r17 [0..7]
		movw	r2,r16
;i2c start
		rcall	twi_send_wr_start										; START + ADDR	
;i2c write Control Byte CMD STREAM
		ldi		r16,(SSD1306_CONTROL_BYTE_CMD_STREAM)					; CONTROL Byte: CMD STREAM
		rcall	twi_write
;i2c write CMD
		andi	r17,0x07
		ori		r17,(SSD1306_CMD_SET_PAGE_START_ADDRESS)				; SET_PAGE START_ADDRESS / 0xB0..0xB7
		mov		r16,r17
		rcall	twi_write
		
		
;i2c write CMD
		ldi		r16,(SSD1306_CMD_SET_COLUMN_ADDRESS)					; SET_COLUMN_ADDRRSS (0x21)
		rcall	twi_write
		mov		r16,r2
		andi	r16,0x0F
		ldi		r17,8
		mul		r16,r17
;i2c write DATA
		mov		r16,r0					; Start Address
		rcall	twi_write
;i2c write DATA
		ldi		r16,127					; End Address			
		rcall	twi_write
;i2c write CMD
		ldi		r16,(SSD1306_CMD_SET_DISPLAY_START_LINE)+(0)		; SET_DISPLAY_START_LINE
		rcall	twi_write
		rcall	twi_send_stop
		movw	r16,r2
		ret


oled_write_string:	// x=r16 ; y=r17 ; Y=Pointer auf string
		ldi		r18,OLED_ADDR
		lsl		r18
		mov		i2c_addr,r18
		rcall	oled_gotoxy
oled_write_string_loop_a:
		ld		r18,Y+
		tst		r18
		breq	oled_write_string_end
		rcall	oled_write_char
		rjmp	oled_write_string_loop_a
oled_write_string_end:
;i2c stop
		rcall	twi_send_stop
		rcall	oled_home
		ret
		

oled_write_fstring:		// x=r16 ; y=r17 ; Z=Pointer auf string		
		ldi		r18,OLED_ADDR
		lsl		r18
		mov		i2c_addr,r18
		rcall	oled_gotoxy
oled_write_fstring_loop_a:
		lpm		r18,Z+
		tst		r18
		breq	oled_write_fstring_end
		rcall	oled_write_char
		rjmp	oled_write_fstring_loop_a
oled_write_fstring_end:
;i2c stop
		rcall	twi_send_stop
		rcall	oled_home

		ret

oled_write_char:
		movw	r24,r30	
		subi	r18,' '
		ldi		r19,(8)
		mul		r18,r19
		load_p	Z,(SSD1306_FONT_8x8)*2
		add		ZL,r0
		adc		ZH,r1
;i2c start
		rcall	twi_send_wr_start										; START + ADDR
;i2c write Control Byte -> DATA STREAM
		ldi		r16,(SSD1306_CONTROL_BYTE_DATA_STREAM)					; CONTROL Byte: DATA STREAM
		rcall	twi_write
oled_char_loop_a:
;i2c write DATA
		lpm		r16,Z+
		rcall	twi_write
		dec		r19
		brne	oled_char_loop_a
;i2c stop
		rcall	twi_send_stop
		movw	r30,r24
		ret


SSD1306_FONT_8x8:
.db		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00		// 0x20
.db		0x00,0x06,0x5F,0x5F,0x06,0x00,0x00,0x00		// 0x21
.db		0x00,0x07,0x07,0x00,0x07,0x07,0x00,0x00		// 0x22
.db		0x14,0x7F,0x7F,0x14,0x7F,0x7F,0x14,0x00		// 0x23
.db		0x24,0x2E,0x6B,0x6B,0x3A,0x12,0x00,0x00		// 0x24
;.db		0x42,0x25,0x12,0x08,0x24,0x52,0x21,0x00		// 0x25		%
.db		0x47,0x65,0x37,0x18,0x7C,0x56,0x73,0x00		// 0x25		%
;.db		0x46,0x66,0x30,0x18,0x0C,0x66,0x62,0x00		// 0x25
.db		0x30,0x7A,0x4F,0x5D,0x37,0x7A,0x48,0x00		// 0x26
.db		0x00,0x00,0x06,0x0F,0x09,0x0F,0x06,0x00		// 0x27
;.db		0x04,0x07,0x03,0x00,0x00,0x00,0x00,0x00		// 0x27	'
.db		0x00,0x1C,0x3E,0x63,0x41,0x00,0x00,0x00		// 0x28
.db		0x00,0x41,0x63,0x3E,0x1C,0x00,0x00,0x00		// 0x29
.db		0x08,0x2A,0x3E,0x1C,0x1C,0x3E,0x2A,0x08		// 0x2A
.db		0x08,0x08,0x3E,0x3E,0x08,0x08,0x00,0x00		// 0x2B
.db		0x00,0xA0,0xE0,0x60,0x00,0x00,0x00,0x00		// 0x2C
.db		0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x00		// 0x2D
.db		0x00,0x00,0x60,0x60,0x00,0x00,0x00,0x00		// 0x2E
.db		0x60,0x30,0x18,0x0C,0x06,0x03,0x01,0x00		// 0x2F
.db		0x3E,0x7F,0x59,0x4D,0x7F,0x3E,0x00,0x00		// 0x30
.db		0x42,0x42,0x7F,0x7F,0x40,0x40,0x00,0x00		// 0x31
.db		0x62,0x73,0x59,0x49,0x6F,0x66,0x00,0x00		// 0x32
.db		0x22,0x63,0x49,0x49,0x7F,0x36,0x00,0x00		// 0x33
.db		0x18,0x1C,0x16,0x13,0x7F,0x7F,0x10,0x00		// 0x34
.db		0x27,0x67,0x45,0x45,0x7D,0x39,0x00,0x00		// 0x35
.db		0x3C,0x7E,0x4B,0x49,0x79,0x30,0x00,0x00		// 0x36
.db		0x03,0x63,0x71,0x19,0x0F,0x07,0x00,0x00		// 0x37
.db		0x36,0x7F,0x49,0x49,0x7F,0x36,0x00,0x00		// 0x38
.db		0x06,0x4F,0x49,0x69,0x3F,0x1E,0x00,0x00		// 0x39
.db		0x00,0x00,0x6C,0x6C,0x00,0x00,0x00,0x00		// 0x3A
.db		0x00,0xA0,0xEC,0x6C,0x00,0x00,0x00,0x00		// 0x3B
.db		0x08,0x1C,0x36,0x63,0x41,0x00,0x00,0x00		// 0x3C
.db		0x14,0x14,0x14,0x14,0x14,0x14,0x00,0x00		// 0x3D
.db		0x00,0x41,0x63,0x36,0x1C,0x08,0x00,0x00		// 0x3E
.db		0x02,0x03,0x51,0x59,0x0F,0x06,0x00,0x00		// 0x3F
.db		0x3E,0x7F,0x41,0x5D,0x5D,0x1F,0x1E,0x00		// 0x40
.db		0x7C,0x7E,0x13,0x13,0x7E,0x7C,0x00,0x00		// 0x41
.db		0x41,0x7F,0x7F,0x49,0x49,0x7F,0x36,0x00		// 0x42
.db		0x1C,0x3E,0x63,0x41,0x41,0x63,0x22,0x00		// 0x43
.db		0x41,0x7F,0x7F,0x41,0x63,0x7F,0x1C,0x00		// 0x44
.db		0x41,0x7F,0x7F,0x49,0x5D,0x41,0x63,0x00		// 0x45
.db		0x41,0x7F,0x7F,0x49,0x1D,0x01,0x03,0x00		// 0x46
.db		0x1C,0x3E,0x63,0x41,0x51,0x73,0x72,0x00		// 0x47
.db		0x7F,0x7F,0x08,0x08,0x7F,0x7F,0x00,0x00		// 0x48
.db		0x00,0x41,0x7F,0x7F,0x41,0x00,0x00,0x00		// 0x49
.db		0x30,0x70,0x40,0x41,0x7F,0x3F,0x01,0x00		// 0x4A
.db		0x41,0x7F,0x7F,0x08,0x1C,0x77,0x63,0x00		// 0x4B
.db		0x41,0x7F,0x7F,0x41,0x40,0x60,0x70,0x00		// 0x4C
.db		0x7F,0x7F,0x06,0x0C,0x06,0x7F,0x7F,0x00		// 0x4D
.db		0x7F,0x7F,0x06,0x0C,0x18,0x7F,0x7F,0x00		// 0x4E
.db		0x1C,0x3E,0x63,0x41,0x63,0x3E,0x1C,0x00		// 0x4F
.db		0x41,0x7F,0x7F,0x49,0x09,0x0F,0x06,0x00		// 0x50
.db		0x1E,0x3F,0x21,0x71,0x7F,0x5E,0x00,0x00		// 0x51
.db		0x41,0x7F,0x7F,0x19,0x39,0x6F,0x46,0x00		// 0x52
.db		0x26,0x67,0x4D,0x59,0x7B,0x32,0x00,0x00		// 0x53
.db		0x03,0x41,0x7F,0x7F,0x41,0x03,0x00,0x00		// 0x54
.db		0x7F,0x7F,0x40,0x40,0x7F,0x7F,0x00,0x00		// 0x55
.db		0x1F,0x3F,0x60,0x60,0x3F,0x1F,0x00,0x00		// 0x56
.db		0x7F,0x7F,0x30,0x18,0x30,0x7F,0x7F,0x00		// 0x57
.db		0x63,0x77,0x1C,0x08,0x1C,0x77,0x63,0x00		// 0x58
.db		0x07,0x4F,0x78,0x78,0x4F,0x07,0x00,0x00		// 0x59
.db		0x67,0x73,0x59,0x4D,0x47,0x63,0x71,0x00		// 0x5A
.db		0x00,0x7F,0x7F,0x41,0x41,0x00,0x00,0x00		// 0x5B
.db		0x01,0x03,0x06,0x0C,0x18,0x30,0x60,0x00		// 0x5C
.db		0x00,0x41,0x41,0x7F,0x7F,0x00,0x00,0x00		// 0x5D
.db		0x08,0x0C,0x06,0x03,0x06,0x0C,0x08,0x00		// 0x5E
.db		0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80		// 0x5F
.db		0x00,0x00,0x03,0x07,0x04,0x00,0x00,0x00		// 0x60
.db		0x20,0x74,0x54,0x54,0x3C,0x78,0x40,0x00		// 0x61
.db		0x41,0x3F,0x7F,0x44,0x44,0x7C,0x38,0x00		// 0x62
.db		0x38,0x7C,0x44,0x44,0x6C,0x28,0x00,0x00		// 0x63
.db		0x30,0x78,0x48,0x49,0x3F,0x7F,0x40,0x00		// 0x64
.db		0x38,0x7C,0x54,0x54,0x5C,0x18,0x00,0x00		// 0x65
.db		0x48,0x7E,0x7F,0x49,0x03,0x02,0x00,0x00		// 0x66
.db		0x98,0xBC,0xA4,0xA4,0xF8,0x7C,0x04,0x00		// 0x67
.db		0x41,0x7F,0x7F,0x08,0x04,0x7C,0x78,0x00		// 0x68
.db		0x00,0x44,0x7D,0x7D,0x40,0x00,0x00,0x00		// 0x69
.db		0x40,0xC4,0x84,0xFD,0x7D,0x00,0x00,0x00		// 0x6A
.db		0x41,0x7F,0x7F,0x10,0x38,0x6C,0x44,0x00		// 0x6B
.db		0x00,0x41,0x7F,0x7F,0x40,0x00,0x00,0x00		// 0x6C
.db		0x7C,0x7C,0x0C,0x18,0x0C,0x7C,0x78,0x00		// 0x6D
.db		0x7C,0x7C,0x04,0x04,0x7C,0x78,0x00,0x00		// 0x6E
.db		0x38,0x7C,0x44,0x44,0x7C,0x38,0x00,0x00		// 0x6F
.db		0x84,0xFC,0xF8,0xA4,0x24,0x3C,0x18,0x00		// 0x70
.db		0x18,0x3C,0x24,0xA4,0xF8,0xFC,0x84,0x00		// 0x71
.db		0x44,0x7C,0x78,0x44,0x1C,0x18,0x00,0x00		// 0x72
.db		0x48,0x5C,0x54,0x54,0x74,0x24,0x00,0x00		// 0x73
.db		0x00,0x04,0x3E,0x7F,0x44,0x24,0x00,0x00		// 0x74
.db		0x3C,0x7C,0x40,0x40,0x3C,0x7C,0x40,0x00		// 0x75
.db		0x1C,0x3C,0x60,0x60,0x3C,0x1C,0x00,0x00		// 0x76
.db		0x3C,0x7C,0x60,0x30,0x60,0x7C,0x3C,0x00		// 0x77
.db		0x44,0x6C,0x38,0x10,0x38,0x6C,0x44,0x00		// 0x78
.db		0x9C,0xBC,0xA0,0xA0,0xFC,0x7C,0x00,0x00		// 0x79
.db		0x4C,0x64,0x74,0x5C,0x4C,0x64,0x00,0x00		// 0x7A
.db		0x08,0x08,0x3E,0x77,0x41,0x41,0x00,0x00		// 0x7B
.db		0x00,0x00,0x00,0x77,0x77,0x00,0x00,0x00		// 0x7C
.db		0x41,0x41,0x77,0x3E,0x08,0x08,0x00,0x00		// 0x7D
.db		0x02,0x03,0x01,0x03,0x02,0x03,0x01,0x00		// 0x7E
.db		0x78,0x7C,0x46,0x43,0x46,0x7C,0x78,0x00		// 0x7F
