;
; m8_BME280_oLED.asm
;
; Created: 		25.11.2020 13:27:23
; Author : 		Steffen Hoerhold
; MPU    :		ATmega8


.include	"macros.inc"

.equ		F_CPU				= 16000000		; Taktfrequenz der CPU in MHz


; BME280 settings
.equ		BME280_ADDR			= 0x76
#define		bme280_t_sb			bme280_t_sb_125ms		;bme280_t_sb_500ms
#define		bme280_mode			bme280_normal_mode		;bme280_forced_mode
#define		bme280_filter		bme280_filter_x4
#define		bme280_osrs_t		bme280_osrs_t_x4
#define		bme280_osrs_p		bme280_osrs_p_x2
#define		bme280_osrs_h		bme280_osrs_h_x16


; TWI settings
.equ	TWI_PORT			= PORTC
.equ	TWI_SCL				= PC5
.equ	TWI_SDA				= PC4
.equ	F_TWI				= 400000		; Hz

.equ	secondLED_PORT		= PORTB
.equ	secondLED_PIN		= 0




.def	i2c_addr		= r12
.def	ZERO			= r13
.def	ONE				= r14
.def	iSREG			= r15
.def	iTEMPA			= r24



.include	"sram_variables.inc"


.include	"m8_INT_vectors.inc"
.include	"int24_to_ascii.inc"
.include	"muls16x16_32.inc"
.include	"muls24x16_40.inc"
.include	"muls24x24_48.inc"
.include	"muls32x8_40.inc"
.include	"muls32x16_48.inc"
.include	"muls32x32_64.inc"
.include	"mulsu32x8_40.inc"
.include	"mulsu32x16_48.inc"
.include	"div_u32_u32.inc"
.include	"hw_twi.inc"
.include	"mbme280.inc"
.include	"bme280_math.inc"
.include	"delay.inc"
.include	"mSSD1306_driver.inc"

; Replace with your application code





RESET:
;STACK initialisieren
		ldi		r16,low(RAMEND)	; Stackpointer initialisieren
		out		SPL,r16			; muss in jedes Programm 
		ldi		r16,high(RAMEND)	; als erstes mit eingebunden werden
		out		SPH,r16
		clr		ZERO
		ldi		r16,1
		mov		ONE,r16


		ldi		YL,Byte1(SRAM_SIZE)
		ldi		YH,Byte2(SRAM_SIZE)
		load_p	Z,(SRAM_START)
flush_sRAM:
		st		Z+,zero
		sbiw	Y,1
		brne	flush_sRAM

		sbi		(secondLED_PORT-1),secondLED_PIN				; PORT.DIR
		sbi		(secondLED_PORT-0),secondLED_PIN				; PORT.OUT
		sbi		PORTC,PC3
		sbi		PORTC-1,PC3										; PORT.DIR
		sbi		PORTB-1,PB1										; PORT.DIR
		cbi		PORTB,PB1										; PORT.OUT
		sei


		rcall	init_twi
		rcall	init_bme280
		rcall	bme280_read_coefficients
		rcall	init_oled
		rcall	oled_clr_screen

		ldi		r16,11
		ldi		r17,1
		load_p	Z,(otxt_temp_unit)*2
		rcall	oled_write_fstring
		ldi		r16,11
		ldi		r17,4
		load_p	Z,(otxt_press_unit)*2
		rcall	oled_write_fstring
		ldi		r16,11
		ldi		r17,7
		load_p	Z,(otxt_hum_unit)*2
		rcall	oled_write_fstring

send_data:
		rcall	bme280_forced_read_raw_data

		rcall	bme280_calc_temperatur
		rcall	bme280_calc_pressure
		rcall	bme280_calc_humidity

		rcall	oled_show_sensor_data

		delayms	1000

		
		sbic	(secondLED_PORT-2),(secondLED_PIN)				; PORT.IN
		cbi		(secondLED_PORT-0),secondLED_PIN					; PORT.OUT
		sbis	(secondLED_PORT-2),(secondLED_PIN)				; PORT.IN
		sbi		(secondLED_PORT-0),secondLED_PIN					; PORT.OUT

		rjmp	send_data



stop:
; Replace with your application code
start:
    inc r16
    rjmp start





;*******************************************************************
copy_var32:
		load_p	X,(ramBUFFER_C)
		ld		r16,Y+
		st		X+,r16
		ld		r16,Y+
		st		X+,r16
		ld		r16,Y+
		st		X+,r16
		ld		r16,Y+
		st		X+,r16
		ret

oled_show_sensor_data:
;*******************************************************************
		load_p	Y,(ramBME280_TEMP)
		rcall	copy_var32
		load_p	Y,(ramBUFFER_C)
		rcall	int24s_ascii
		sts		((BCD_VALUE)+10),zero
		lds		r16,((BCD_VALUE)+8)
		sts		((BCD_VALUE)+9),r16
		lds		r16,((BCD_VALUE)+7)
		sts		((BCD_VALUE)+8),r16
		ldi		r16,'.'
		sts		((BCD_VALUE)+7),r16		
		ldi		r16,0
		ldi		r17,1
		load_p	Y,(BCD_VALUE)
		rcall	oled_write_string
;*******************************************************************
		load_p	Y,(ramBME280_PRESSURE)
;		rcall	copy_var32
		ld		r16,Y+
		ld		r17,Y+
		ld		r18,Y+
		ld		r19,Y+
		subi	r16,BYTE1(-1000)
		sbci	r17,BYTE2(-1000)
		sbci	r18,BYTE3(-1000)
		sbci	r19,BYTE4(-1000)
		load_p	Y,(ramBUFFER_C)
		st		Y+,r16
		st		Y+,r17
		st		Y+,r18
		st		Y+,r19
		load_p	Y,(ramBUFFER_C)
		rcall	int24s_ascii
		sts		((BCD_VALUE)+10),zero
		lds		r16,((BCD_VALUE)+8)
		sts		((BCD_VALUE)+9),r16
		lds		r16,((BCD_VALUE)+7)
		sts		((BCD_VALUE)+8),r16
		ldi		r16,'.'
		sts		((BCD_VALUE)+7),r16		
		ldi		r16,0
		ldi		r17,4
		load_p	Y,(BCD_VALUE)
		rcall	oled_write_string
;*******************************************************************
		load_p	Y,(ramBME280_HUMIDITY)
		rcall	copy_var32
		load_p	Y,(ramBUFFER_C)
		rcall	int24s_ascii
		sts		((BCD_VALUE)+9),zero
		lds		r16,((BCD_VALUE)+7)
		sts		((BCD_VALUE)+8),r16
		lds		r16,((BCD_VALUE)+6)
		sts		((BCD_VALUE)+7),r16
		ldi		r16,'.'
		sts		((BCD_VALUE)+6),r16
		ldi		r16,1
		ldi		r17,7
		load_p	Y,(BCD_VALUE)
		rcall	oled_write_string
		ret
;*******************************************************************




otxt_temp:
.db		"Temperatur",0,0
otxt_press:
.db		"Luftdruck",0
otxt_hum:
.db		"Luftfeuchte",0

otxt_temp_unit:
.db		"'C",0,0
otxt_press_unit:
.db		"hPa",0
otxt_hum_unit:
.db		"%rH",0
